import { GArray, Node, Node3D, PackedScene, Signal1, Timer, Vector3 } from 'godot'
import { export_file, signal } from 'godot.annotations'
import { export_node, onready } from './annotations'
import Weapon from './weapon'
import AsyncResourceLoader from './async_resource_loader'

export default class EquippedWeapon extends Node {
  @export_file('*.tres')
  starting_weapon?: string
  _starting_weapon?: Weapon

  @export_node()
  transform_parent!: Node

  _equipped_weapon?: Weapon
  _has_equipped_weapon: boolean = false

  @signal()
  equipped!: Signal1<Weapon>

  @signal()
  unequipped!: Signal1<Weapon>

  @onready('FireRate')
  _fire_rate_timer!: Timer

  has_equipped_weapon(): boolean {
    return this._has_equipped_weapon
  }

  _ready(): void {
    if (this.starting_weapon != null) {
      AsyncResourceLoader.instance.load<Weapon>(this.starting_weapon, 'Weapon').then(weapon => this.equip(weapon))
    }
  }

  unequip() {
    if (this._equipped_weapon != null) {
      const previous = this._equipped_weapon
      this._equipped_weapon = undefined
      this._has_equipped_weapon = false
      this.unequipped.emit(previous)
    }
  }

  _parent_scene_to_transform(scene: PackedScene) {
    const children: GArray<Node> = this.transform_parent.get_children()
    for (const child of children) {
      child.queue_free()
    }

    const node = scene.instantiate() as Node3D
    this.add_child(node)
    node.reparent(this.transform_parent, false)
  }

  equip(weapon: Weapon) {
    AsyncResourceLoader.instance.load<PackedScene>(weapon.scene, 'PackedScene')
      .then(scene => this._parent_scene_to_transform(scene))

    if (this._has_equipped_weapon) {
      this.unequip()
    }

    this._has_equipped_weapon = true
    this._equipped_weapon = weapon
    this._fire_rate_timer.wait_time = weapon.fire_rate
    this.equipped.emit(weapon)
  }

  fire(): boolean {
    if (this._fire_rate_timer.time_left <= 0) {
      this._fire_rate_timer.start()
      return true
    } else {
      return false
    }
  }
}

